@@ -22,43 +22,55 @@ import copy
 import functools
 import warnings
 
+from keystoneauth1 import exceptions as ksa_exceptions
+from keystoneauth1 import plugin
 from oslo_utils import strutils
 import six
 from six.moves import urllib
 
-from keystoneclient import auth
-from keystoneclient import exceptions
+from keystoneclient import exceptions as ksc_exceptions
 from keystoneclient.i18n import _
 
 
+class Response(object):
+
+    def __init__(self, http_response, data):
+        self.request_ids = []
+        if isinstance(http_response, list):
+            # http_response is a list of <requests.Response> in case
+            # of pagination
+            for resp_obj in http_response:
+                # Extract 'x-openstack-request-id' from headers
+                self.request_ids.append(resp_obj.headers.get(
+                    'x-openstack-request-id'))
+        else:
+            self.request_ids.append(http_response.headers.get(
+                'x-openstack-request-id'))
+        self.data = data
+
+
 def getid(obj):
     """Return id if argument is a Resource.
 
     Abstracts the common pattern of allowing both an object or an object's ID
     (UUID) as a parameter when dealing with relationships.
     """
-    try:
-        if obj.uuid:
-            return obj.uuid
-    except AttributeError:  # nosec(cjschaef): 'obj' doesn't contain attribute
-        # 'uuid', return attribute 'id' or the 'obj'
-        pass
-    try:
-        return obj.id
-    except AttributeError:
-        return obj
+    if getattr(obj, 'uuid', None):
+        return obj.uuid
+    else:
+        return getattr(obj, 'id', obj)
 
 
 def filter_none(**kwargs):
     """Remove any entries from a dictionary where the value is None."""
-    return dict((k, v) for k, v in six.iteritems(kwargs) if v is not None)
+    return dict((k, v) for k, v in kwargs.items() if v is not None)
 
 
 def filter_kwargs(f):
     @functools.wraps(f)
     def func(*args, **kwargs):
         new_kwargs = {}
-        for key, ref in six.iteritems(kwargs):
+        for key, ref in kwargs.items():
             if ref is None:
                 # drop null values
                 continue
@@ -106,6 +118,11 @@ class Manager(object):
             'may be removed in the 2.0.0 release', DeprecationWarning)
         return self.client
 
+    def _prepare_return_value(self, http_response, data):
+        if self.client.include_metadata:
+            return Response(http_response, data)
+        return data
+
     def _list(self, url, response_key, obj_class=None, body=None, **kwargs):
         """List the collection.
 
@@ -136,7 +153,8 @@ class Manager(object):
             # are already returned in a list (so simply utilize that list)
             pass
 
-        return [obj_class(self, res, loaded=True) for res in data if res]
+        return self._prepare_return_value(
+            resp, [obj_class(self, res, loaded=True) for res in data if res])
 
     def _get(self, url, response_key, **kwargs):
         """Get an object from collection.
@@ -147,7 +165,8 @@ class Manager(object):
         :param kwargs: Additional arguments will be passed to the request.
         """
         resp, body = self.client.get(url, **kwargs)
-        return self.resource_class(self, body[response_key], loaded=True)
+        return self._prepare_return_value(
+            resp, self.resource_class(self, body[response_key], loaded=True))
 
     def _head(self, url, **kwargs):
         """Retrieve request headers for an object.
@@ -156,7 +175,7 @@ class Manager(object):
         :param kwargs: Additional arguments will be passed to the request.
         """
         resp, body = self.client.head(url, **kwargs)
-        return resp.status_code == 204
+        return self._prepare_return_value(resp, resp.status_code == 204)
 
     def _post(self, url, body, response_key, return_raw=False, **kwargs):
         """Create an object.
@@ -173,7 +192,8 @@ class Manager(object):
         resp, body = self.client.post(url, body=body, **kwargs)
         if return_raw:
             return body[response_key]
-        return self.resource_class(self, body[response_key])
+        return self._prepare_return_value(
+            resp, self.resource_class(self, body[response_key]))
 
     def _put(self, url, body=None, response_key=None, **kwargs):
         """Update an object with PUT method.
@@ -189,9 +209,15 @@ class Manager(object):
         # PUT requests may not return a body
         if body is not None:
             if response_key is not None:
-                return self.resource_class(self, body[response_key])
+                return self._prepare_return_value(
+                    resp, self.resource_class(self, body[response_key]))
             else:
-                return self.resource_class(self, body)
+                return self._prepare_return_value(
+                    resp, self.resource_class(self, body))
+        # In some cases (e.g. 'add_endpoint_to_project' from endpoint_filters
+        # resource), PUT request may not return a body so return None as
+        # response along with request_id if include_metadata is True.
+        return self._prepare_return_value(resp, body)
 
     def _patch(self, url, body=None, response_key=None, **kwargs):
         """Update an object with PATCH method.
@@ -205,9 +231,11 @@ class Manager(object):
         """
         resp, body = self.client.patch(url, body=body, **kwargs)
         if response_key is not None:
-            return self.resource_class(self, body[response_key])
+            return self._prepare_return_value(
+                resp, self.resource_class(self, body[response_key]))
         else:
-            return self.resource_class(self, body)
+            return self._prepare_return_value(
+                resp, self.resource_class(self, body))
 
     def _delete(self, url, **kwargs):
         """Delete an object.
@@ -215,7 +243,8 @@ class Manager(object):
         :param url: a partial URL, e.g., '/servers/my-server'
         :param kwargs: Additional arguments will be passed to the request.
         """
-        return self.client.delete(url, **kwargs)
+        resp, body = self.client.delete(url, **kwargs)
+        return resp, self._prepare_return_value(resp, body)
 
     def _update(self, url, body=None, response_key=None, method="PUT",
                 **kwargs):
@@ -226,11 +255,14 @@ class Manager(object):
             resp, body = methods[method](url, body=body,
                                          **kwargs)
         except KeyError:
-            raise exceptions.ClientException(_("Invalid update method: %s")
-                                             % method)
+            raise ksc_exceptions.ClientException(_("Invalid update method: %s")
+                                                 % method)
         # PUT requests may not return a body
         if body:
-            return self.resource_class(self, body[response_key])
+            return self._prepare_return_value(
+                resp, self.resource_class(self, body[response_key]))
+        else:
+            return self._prepare_return_value(resp, body)
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -248,16 +280,20 @@ class ManagerWithFind(Manager):
         the Python side.
         """
         rl = self.findall(**kwargs)
-        num = len(rl)
 
-        if num == 0:
+        if self.client.include_metadata:
+            base_response = rl
+            rl = rl.data
+            base_response.data = rl[0]
+
+        if len(rl) == 0:
             msg = _("No %(name)s matching %(kwargs)s.") % {
                 'name': self.resource_class.__name__, 'kwargs': kwargs}
-            raise exceptions.NotFound(404, msg)
-        elif num > 1:
-            raise exceptions.NoUniqueMatch
+            raise ksa_exceptions.NotFound(404, msg)
+        elif len(rl) > 1:
+            raise ksc_exceptions.NoUniqueMatch
         else:
-            return rl[0]
+            return base_response if self.client.include_metadata else rl[0]
 
     def findall(self, **kwargs):
         """Find all items with attributes matching ``**kwargs``.
@@ -268,15 +304,23 @@ class ManagerWithFind(Manager):
         found = []
         searches = kwargs.items()
 
-        for obj in self.list():
-            try:
-                if all(getattr(obj, attr) == value
-                       for (attr, value) in searches):
-                    found.append(obj)
-            except AttributeError:
-                continue
+        def _extract_data(objs, response_data):
+            for obj in objs:
+                try:
+                    if all(getattr(obj, attr) == value
+                           for (attr, value) in searches):
+                        response_data.append(obj)
+                except AttributeError:
+                    continue
+            return response_data
 
-        return found
+        objs = self.list()
+        if self.client.include_metadata:
+            # 'objs' is the object of 'Response' class.
+            objs.data = _extract_data(objs.data, found)
+            return objs
+
+        return _extract_data(objs, found)
 
 
 class CrudManager(Manager):
@@ -352,7 +396,17 @@ class CrudManager(Manager):
         return self._head(self.build_url(dict_args_in_out=kwargs))
 
     def _build_query(self, params):
-        return '?%s' % urllib.parse.urlencode(params) if params else ''
+        if params is None:
+            return ''
+        else:
+            # NOTE(spilla) Since the manager cannot take in a hyphen as a
+            # key in the kwarg, it is passed in with a _.  This needs to be
+            # replaced with a proper hyphen for the URL to work properly.
+            tags_params = ('tags_any', 'not_tags', 'not_tags_any')
+            for tag_param in tags_params:
+                if tag_param in params:
+                    params[tag_param.replace('_', '-')] = params.pop(tag_param)
+            return '?%s' % urllib.parse.urlencode(params, doseq=True)
 
     def build_key_only_query(self, params_list):
         """Build a query that does not include values, just keys.
@@ -365,6 +419,16 @@ class CrudManager(Manager):
 
     @filter_kwargs
     def list(self, fallback_to_auth=False, **kwargs):
+
+        def return_resp(resp, include_metadata=False):
+            base_response = None
+            list_data = resp
+            if include_metadata:
+                base_response = resp
+                list_data = resp.data
+                base_response.data = list_data
+            return base_response if include_metadata else list_data
+
         if 'id' in kwargs.keys():
             # Ensure that users are not trying to call things like
             # ``domains.list(id='default')`` when they should have used
@@ -381,15 +445,16 @@ class CrudManager(Manager):
         try:
             query = self._build_query(kwargs)
             url_query = '%(url)s%(query)s' % {'url': url, 'query': query}
-            return self._list(
-                url_query,
-                self.collection_key)
-        except exceptions.EmptyCatalog:
+            list_resp = self._list(url_query, self.collection_key)
+            return return_resp(list_resp,
+                               include_metadata=self.client.include_metadata)
+        except ksa_exceptions.EmptyCatalog:
             if fallback_to_auth:
-                return self._list(
-                    url_query,
-                    self.collection_key,
-                    endpoint_filter={'interface': auth.AUTH_INTERFACE})
+                list_resp = self._list(url_query, self.collection_key,
+                                       endpoint_filter={
+                                           'interface': plugin.AUTH_INTERFACE})
+                return return_resp(
+                    list_resp, include_metadata=self.client.include_metadata)
             else:
                 raise
 
@@ -420,22 +485,28 @@ class CrudManager(Manager):
         url = self.build_url(dict_args_in_out=kwargs)
 
         query = self._build_query(kwargs)
-        rl = self._list(
-            '%(url)s%(query)s' % {
-                'url': url,
-                'query': query,
-            },
+        url_query = '%(url)s%(query)s' % {
+            'url': url,
+            'query': query
+        }
+        elements = self._list(
+            url_query,
             self.collection_key)
-        num = len(rl)
 
-        if num == 0:
+        if self.client.include_metadata:
+            base_response = elements
+            elements = elements.data
+            base_response.data = elements[0]
+
+        if not elements:
             msg = _("No %(name)s matching %(kwargs)s.") % {
                 'name': self.resource_class.__name__, 'kwargs': kwargs}
-            raise exceptions.NotFound(404, msg)
-        elif num > 1:
-            raise exceptions.NoUniqueMatch
+            raise ksa_exceptions.NotFound(404, msg)
+        elif len(elements) > 1:
+            raise ksc_exceptions.NoUniqueMatch
         else:
-            return rl[0]
+            return (base_response if self.client.include_metadata
+                    else elements[0])
 
 
 class Resource(object):
@@ -477,16 +548,23 @@ class Resource(object):
         return None
 
     def _add_details(self, info):
-        for (k, v) in six.iteritems(info):
+        for (k, v) in info.items():
             try:
-                setattr(self, k, v)
+                try:
+                    setattr(self, k, v)
+                except UnicodeEncodeError:
+                    # This happens when we're running with Python version that
+                    # does not support Unicode identifiers (e.g. Python 2.7).
+                    # In that case we can't help but not set this attribute;
+                    # it'll be available in a dict representation though
+                    pass
                 self._info[k] = v
             except AttributeError:  # nosec(cjschaef): we already defined the
                 # attribute on the class
                 pass
 
     def __getattr__(self, k):
-        """Checking attrbiute existence."""
+        """Checking attribute existence."""
         if k not in self.__dict__:
             # NOTE(bcwaldon): disallow lazy-loading if already loaded once
             if not self.is_loaded():
@@ -511,8 +589,6 @@ class Resource(object):
         new = self.manager.get(self.id)
         if new:
             self._add_details(new._info)
-            self._add_details(
-                {'x_request_id': self.manager.client.last_request_id})
 
     def __eq__(self, other):
         """Define equality for resources."""
@@ -523,6 +599,10 @@ class Resource(object):
             return False
         return self._info == other._info
 
+    def __ne__(self, other):
+        """Define inequality for resources."""
+        return not self == other
+
     def is_loaded(self):
         return self._loaded
 
