@@ -15,30 +15,35 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-"""
-OpenStack Client interface. Handles the REST calls and responses.
-"""
+"""OpenStack Client interface. Handles the REST calls and responses."""
 
 import logging
 import warnings
 
+try:
+    # For Python 3.8 and later
+    import importlib.metadata as importlib_metadata
+except ImportError:
+    # For everyone else
+    import importlib_metadata
+
 from debtcollector import removals
 from debtcollector import renames
 from keystoneauth1 import adapter
 from oslo_serialization import jsonutils
-import pkg_resources
-from positional import positional
+import packaging.version
 import requests
 
 try:
-    import pickle
+    import pickle  # nosec(cjschaef): see bug 1534288 for details
 
     # NOTE(sdague): The conditional keyring import needs to only
     # trigger if it's a version of keyring that's supported in global
     # requirements. Update _min and _bad when that changes.
-    keyring_v = pkg_resources.parse_version(
-        pkg_resources.get_distribution("keyring").version)
-    keyring_min = pkg_resources.parse_version('5.5.1')
+    keyring_v = packaging.version.Version(
+        importlib_metadata.version('keyring')
+    )
+    keyring_min = packaging.version.Version('5.5.1')
     # This is a list of versions, e.g., pkg_resources.parse_version('3.3')
     keyring_bad = []
 
@@ -46,7 +51,7 @@ try:
         import keyring
     else:
         keyring = None
-except (ImportError, pkg_resources.DistributionNotFound):
+except (ImportError, importlib_metadata.PackageNotFoundError):
     keyring = None
     pickle = None
 
@@ -56,7 +61,7 @@ from keystoneclient import access
 from keystoneclient.auth import base
 from keystoneclient import baseclient
 from keystoneclient import exceptions
-from keystoneclient.i18n import _, _LW
+from keystoneclient.i18n import _
 from keystoneclient import session as client_session
 
 
@@ -129,23 +134,25 @@ class _KeystoneAdapter(adapter.LegacyJsonAdapter):
         # the identity plugin case
         try:
             return self.session.auth.get_access(self.session).user_id
-        except AttributeError:
+        except AttributeError:  # nosec(cjschaef): attempt legacy retrival, or
+            # return None
             pass
 
-        # there is a case that we explicity allow (tested by our unit tests)
+        # there is a case that we explicitly allow (tested by our unit tests)
         # that says you should be able to set the user_id on a legacy client
         # and it should overwrite the one retrieved via authentication. If it's
         # a legacy then self.session.auth is a client and we retrieve user_id.
         try:
             return self.session.auth.user_id
-        except AttributeError:
+        except AttributeError:  # nosec(cjschaef): retrivals failed, return
+            # None
             pass
 
         return None
 
 
 class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
-    """HTTP client
+    """HTTP client.
 
     .. warning::
 
@@ -194,9 +201,6 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
     :param string original_ip: This argument is deprecated as of the 1.7.0
                                release in favor of session and may be removed
                                in the 2.0.0 release. (optional)
-    :param boolean debug: This argument is deprecated as of the 1.7.0 release
-                          in favor of logging configuration and may be removed
-                          in the 2.0.0 release. (optional)
     :param dict auth_ref: To allow for consumers of the client to manage their
                           own caching strategy, you may initialize a client
                           with a previously captured auth_reference (token). If
@@ -225,7 +229,7 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
     :param string service_name: The default service_name for URL discovery.
                                 default: None (optional)
     :param string interface: The default interface for URL discovery.
-                             default: admin (optional)
+                             default: admin (v2), public (v3). (optional)
     :param string endpoint_override: Always use this endpoint URL for requests
                                      for this client. (optional)
     :param auth: An auth plugin to use instead of the session one. (optional)
@@ -244,16 +248,15 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
                            removal_version='2.0.0')
     @renames.renamed_kwarg('tenant_id', 'project_id', version='1.7.0',
                            removal_version='2.0.0')
-    @positional(enforcement=positional.WARN)
     def __init__(self, username=None, tenant_id=None, tenant_name=None,
                  password=None, auth_url=None, region_name=None, endpoint=None,
-                 token=None, debug=False, auth_ref=None, use_keyring=False,
+                 token=None, auth_ref=None, use_keyring=False,
                  force_new_token=False, stale_duration=None, user_id=None,
                  user_domain_id=None, user_domain_name=None, domain_id=None,
                  domain_name=None, project_id=None, project_name=None,
                  project_domain_id=None, project_domain_name=None,
                  trust_id=None, session=None, service_name=None,
-                 interface='admin', endpoint_override=None, auth=None,
+                 interface='default', endpoint_override=None, auth=None,
                  user_agent=USER_AGENT, connect_retries=None, **kwargs):
         # set baseline defaults
         self.user_id = None
@@ -376,14 +379,22 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
 
         self.session = session
         self.domain = ''
-        self.debug_log = debug
+
+        version = (
+            _discover.normalize_version_number(self.version) if self.version
+            else None)
+
+        # NOTE(frickler): If we know we have v3, use the public interface as
+        # default, otherwise keep the historic default of admin
+        if interface == 'default':
+            if version == (3, 0):
+                interface = 'public'
+            else:
+                interface = 'admin'
 
         # NOTE(jamielennox): unfortunately we can't just use **kwargs here as
         # it would incompatibly limit the kwargs that can be passed to __init__
         # try and keep this list in sync with adapter.Adapter.__init__
-        version = (
-            _discover.normalize_version_number(self.version) if self.version
-            else None)
         self._adapter = _KeystoneAdapter(session,
                                          service_type='identity',
                                          service_name=service_name,
@@ -395,9 +406,14 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
                                          user_agent=user_agent,
                                          connect_retries=connect_retries)
 
+        # NOTE(dstanek): This allows me to not have to change keystoneauth or
+        # to write an adapter to the adapter here. Splitting thing into
+        # multiple project isn't always all sunshine and roses.
+        self._adapter.include_metadata = kwargs.pop('include_metadata', False)
+
         # keyring setup
         if use_keyring and keyring is None:
-            _logger.warning(_LW('Failed to load keyring modules.'))
+            _logger.warning('Failed to load keyring modules.')
         self.use_keyring = use_keyring and keyring is not None
         self.force_new_token = force_new_token
         self.stale_duration = stale_duration or access.STALE_TOKEN_DURATION
@@ -445,11 +461,14 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
 
     @property
     def service_catalog(self):
-        """Returns this client's service catalog."""
-        return self.auth_ref.service_catalog
+        """Return this client's service catalog."""
+        try:
+            return self.auth_ref.service_catalog
+        except AttributeError:
+            return None
 
     def has_service_catalog(self):
-        """Returns True if this client provides a service catalog."""
+        """Return True if this client provides a service catalog."""
         return self.auth_ref and self.auth_ref.has_service_catalog()
 
     @property
@@ -461,7 +480,6 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
             This is deprecated as of the 1.7.0 release in favor of project_id
             and may be removed in the 2.0.0 release.
         """
-
         warnings.warn(
             'tenant_id is deprecated as of the 1.7.0 release in favor of '
             'project_id and may be removed in the 2.0.0 release.',
@@ -478,7 +496,6 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
             This is deprecated as of the 1.7.0 release in favor of project_name
             and may be removed in the 2.0.0 release.
         """
-
         warnings.warn(
             'tenant_name is deprecated as of the 1.7.0 release in favor of '
             'project_name and may be removed in the 2.0.0 release.',
@@ -486,7 +503,6 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
 
         return self.project_name
 
-    @positional(enforcement=positional.WARN)
     def authenticate(self, username=None, password=None, tenant_name=None,
                      tenant_id=None, auth_url=None, token=None,
                      user_id=None, domain_name=None, domain_id=None,
@@ -606,7 +622,7 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
 
         Used to store and retrieve auth_ref from keyring.
 
-        Returns a slash-separated string of values ordered by key name.
+        Return a slash-separated string of values ordered by key name.
 
         """
         return '/'.join([kwargs[k] or '?' for k in sorted(kwargs)])
@@ -629,28 +645,26 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
                 auth_ref = keyring.get_password("keystoneclient_auth",
                                                 keyring_key)
                 if auth_ref:
-                    auth_ref = pickle.loads(auth_ref)  # nosec
+                    auth_ref = pickle.loads(auth_ref)  # nosec(cjschaef): see
+                    # bug 1534288
                     if auth_ref.will_expire_soon(self.stale_duration):
                         # token has expired, don't use it
                         auth_ref = None
             except Exception as e:
                 auth_ref = None
-                _logger.warning(
-                    _LW('Unable to retrieve token from keyring %s'), e)
+                _logger.warning('Unable to retrieve token from keyring %s', e)
         return (keyring_key, auth_ref)
 
     def store_auth_ref_into_keyring(self, keyring_key):
-        """Store auth_ref into keyring.
-
-        """
+        """Store auth_ref into keyring."""
         if self.use_keyring:
             try:
                 keyring.set_password("keystoneclient_auth",
                                      keyring_key,
-                                     pickle.dumps(self.auth_ref))
+                                     pickle.dumps(self.auth_ref))  # nosec
+                # (cjschaef): see bug 1534288
             except Exception as e:
-                _logger.warning(
-                    _LW("Failed to store token into keyring %s"), e)
+                _logger.warning("Failed to store token into keyring %s", e)
 
     def _process_management_url(self, region_name):
         try:
@@ -658,8 +672,8 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
                 service_type='identity',
                 endpoint_type='admin',
                 region_name=region_name)
-        except exceptions.EndpointNotFound:
-            pass
+        except exceptions.EndpointNotFound as e:
+            _logger.debug("Failed to find endpoint for management url %s", e)
 
     def process_token(self, region_name=None):
         """Extract and process information from the new auth_ref.
@@ -698,7 +712,6 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
         # permanently setting _endpoint would better match that behaviour.
         self._endpoint = value
 
-    @positional(enforcement=positional.WARN)
     def get_raw_token_from_identity_service(self, auth_url, username=None,
                                             password=None, tenant_name=None,
                                             tenant_id=None, token=None,
@@ -747,7 +760,7 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
         return self._adapter.request(*args, **kwargs)
 
     def _cs_request(self, url, method, management=True, **kwargs):
-        """Makes an authenticated request to keystone endpoint.
+        """Make an authenticated request to keystone endpoint.
 
         Request are made to keystone endpoint by concatenating
         self.management_url and url and passing in method and
@@ -870,9 +883,11 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
     deprecated_adapter_variables = {'region_name': None}
 
     def __getattr__(self, name):
+        """Fetch deprecated session variables."""
         try:
             var_name = self.deprecated_session_variables[name]
-        except KeyError:
+        except KeyError:  # nosec(cjschaef): try adapter variable or raise
+            # an AttributeError
             pass
         else:
             warnings.warn(
@@ -883,7 +898,7 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
 
         try:
             var_name = self.deprecated_adapter_variables[name]
-        except KeyError:
+        except KeyError:  # nosec(cjschaef): raise an AttributeError
             pass
         else:
             warnings.warn(
@@ -895,9 +910,11 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
         raise AttributeError(_("Unknown Attribute: %s") % name)
 
     def __setattr__(self, name, val):
+        """Assign value to deprecated seesion variables."""
         try:
             var_name = self.deprecated_session_variables[name]
-        except KeyError:
+        except KeyError:  # nosec(cjschaef): try adapter variable or call
+            # parent class's __setattr__
             pass
         else:
             warnings.warn(
@@ -908,7 +925,7 @@ class HTTPClient(baseclient.Client, base.BaseAuthPlugin):
 
         try:
             var_name = self.deprecated_adapter_variables[name]
-        except KeyError:
+        except KeyError:  # nosec(cjschaef): call parent class's __setattr__
             pass
         else:
             warnings.warn(
