@@ -10,17 +10,20 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import requests
 import uuid
 
-import six
 from six.moves.urllib import parse as urlparse
 
+from keystoneauth1.identity import v3
+from keystoneauth1 import session
 from keystoneclient.tests.unit import client_fixtures
 from keystoneclient.tests.unit import utils
+from keystoneclient.v3 import client
 
 
 def parameterize(ref):
-    """Rewrites attributes to match the kwarg naming convention in client.
+    """Rewrite attributes to match the kwarg naming convention in client.
 
     >>> parameterize({'project_id': 0})
     {'project': 0}
@@ -45,6 +48,7 @@ class UnauthenticatedTestCase(utils.TestCase):
 class TestCase(UnauthenticatedTestCase):
 
     TEST_ADMIN_IDENTITY_ENDPOINT = "http://127.0.0.1:35357/v3"
+    TEST_PUBLIC_IDENTITY_ENDPOINT = "http://127.0.0.1:5000/v3"
 
     TEST_SERVICE_CATALOG = [{
         "endpoints": [{
@@ -222,6 +226,8 @@ class CrudTests(object):
         self.assertRequestBodyIs(json=self.encode(entity))
 
     def test_create(self, ref=None, req_ref=None):
+        deprecations = self.useFixture(client_fixtures.Deprecations())
+        deprecations.expect_deprecations()
         ref = ref or self.new_ref()
         manager_ref = ref.copy()
         manager_ref.pop('id')
@@ -301,7 +307,7 @@ class CrudTests(object):
 
         qs_args = self.requests_mock.last_request.qs
         qs_args_expected = expected_query or filter_kwargs
-        for key, value in six.iteritems(qs_args_expected):
+        for key, value in qs_args_expected.items():
             self.assertIn(key, qs_args)
             # The querystring value is a list. Note we convert the value to a
             # string and lower, as the query string is always a string and the
@@ -344,6 +350,8 @@ class CrudTests(object):
             self.assertQueryStringIs('')
 
     def test_update(self, ref=None, req_ref=None):
+        deprecations = self.useFixture(client_fixtures.Deprecations())
+        deprecations.expect_deprecations()
         ref = ref or self.new_ref()
 
         self.stub_entity('PATCH', id=ref['id'], entity=ref)
@@ -372,3 +380,17 @@ class CrudTests(object):
 
         self.stub_entity('DELETE', id=ref['id'], status_code=204)
         self.manager.delete(ref['id'])
+
+
+class TestRequestId(TestCase):
+    resp = requests.Response()
+    TEST_REQUEST_ID = uuid.uuid4().hex
+    resp.headers['x-openstack-request-id'] = TEST_REQUEST_ID
+
+    def setUp(self):
+        super(TestRequestId, self).setUp()
+        auth = v3.Token(auth_url='http://127.0.0.1:5000',
+                        token=self.TEST_TOKEN)
+        session_ = session.Session(auth=auth)
+        self.client = client.Client(session=session_,
+                                    include_metadata='True')._adapter
